/*****************************************************************************
**  Copyright (C) 2004                                                      **
**  University of Tennessee                                                 **
**                                                                          **
**  See the file COPYRIGHT in the package base directory for details        **
*****************************************************************************/

#include "Callgraph.h"
#include "cone_error.h"

#include <iostream>
   using std::cerr;
   using std::cout;
   using std::endl;

// global variable  
int nodecount = 0 ;

Callgraph::Callgraph(Executable& exec, string mainfun) :
  exec(exec)
{
  for (int i=0; i< exec.get_num_funcs(); i++)
    {
      Function* func = new Function() ;
      int node_index = 0 ;

      // node_id is the id of a cnode 

      int node_id = 0 ;

      // look for the root node i.e main function

      if( exec.get_func(i)->name == mainfun)
        {         
          func = exec.get_func(i) ; 
          Cnode* root = new Cnode(node_index++ , func->csitev[0] , NULL) ;               
 	  root->get_csite_ptr()->cnodev.push_back(root);
	  nodev.push_back(root) ; 

          // look for the children of the root node 
          // hence define level 1         
         
          for (int j=1 ; j < func->csitev.size() ; j++)
            {   
              Cnode* temp = new Cnode(node_index++ , func->csitev[j] , root );
              temp->get_csite_ptr()->cnodev.push_back(temp) ; 
	      nodev.push_back(temp) ;  
              insert_prev_level(temp);              
       	    } 
                

          int temp_size1 = prev_level[0]->get_csite_ptr()->callee->csitev.size(); 
 
          // if atleast one node in the previous level has children, find them 
          while ( ! prev_level_unexpandable() )
            {
              for (int k = 0 ; k < prev_level.size() ; k++)
                {
                  Function* temp_func = new Function() ; 
                  temp_func = prev_level[k]->get_csite_ptr()->callee ; 

                  int temp_size = prev_level[k]->get_csite_ptr()->callee->csitev.size(); 

                  for (int l = 0 ; l < temp_size ; l++) 
                    {  
                      Cnode* temp_node = new Cnode(node_index++ , temp_func->csitev[l] , prev_level[k] );
                      temp_node->get_csite_ptr()->cnodev.push_back(temp_node); 
		      nodev.push_back(temp_node)  ;  
                      insert_next_level(temp_node) ; // insert children of prev_level in new_level
                    }   

                }
        
              prev_level.clear() ;                  
        
              prev_level = next_level ; 
 
              next_level.clear();
                
            } 
        } 
    } 
}


// destructor
Callgraph::~Callgraph()
{
  for(int i = 0; i < nodev.size() ; i++)
    delete nodev[i] ; 
}

// prev_level keeps track of the level to be expanded
void
Callgraph::insert_prev_level(Cnode* cnode_ptr) 
{
  prev_level.push_back(cnode_ptr) ;
}

// next_level keeps track of the level just expanded
void
Callgraph::insert_next_level(Cnode* cnode_ptr) 
{
  next_level.push_back(cnode_ptr) ;
}

// empties theprevious layer
void
Callgraph::empty_prev_level() 
{
  for ( int i = 0; i < prev_level.size(); i++ )
    prev_level.pop_back() ;
}


// prev_level_unexpandable ensures that the level to be expanded has
// some nodes which have children

bool
Callgraph::prev_level_unexpandable()  const
{
  bool is_unexpandable = true ;
  for ( int i=0 ; i < prev_level.size() ; i++)
    {
      if ( prev_level[i]->get_csite_ptr()->callee->csitev.size() != 0  )
        is_unexpandable = false ;    
    } 
  return is_unexpandable ; 
} 

ostream& operator<<(ostream& out, const Callgraph& cgraph)
{ 
  for ( int i = 0; i < cgraph.get_num_nodes(); i++ )
    out <<  *cgraph.get_node(i) ; 
  
  return out;
      
}

 
