/*****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "cone_cndata.h"

#include <stdio.h>
#include<stdlib.h>
#include<string.h>
#include<sys/time.h>
#include<sys/times.h>
#include <sys/resource.h>
#include <unistd.h>

#include "cone_metv.h"


static int met_count;

extern int rank;

#define d sysconf(_SC_CLK_TCK)

struct cone_cndata
{
  int                 cnode_id; 
  int                 nmets;
  unsigned long long  metvals[CONE_METV_MAXSIZE];
  double              time;
  double              u_time;
  double              s_time;
  unsigned long long  enter_vals[CONE_METV_MAXSIZE];
  double              enter_time;  
  double              enter_u_time;
  double              enter_s_time;
  long                visits;
};

struct timeval  tv;
struct timezone tz;
struct rusage   usage;


struct cone_cndata*  cone_cndata_create(int cnode_id)
{
  int i;
  struct cone_cndata* cndata = NULL;

  cndata = (struct cone_cndata*) malloc(sizeof(struct cone_cndata));
  if ( cndata == NULL )
    cone_error();

  cndata->cnode_id = cnode_id;
  cndata->nmets = cone_metv_size();

  cndata->time   = 0.0;
 
  for ( i = 0 ; i < CONE_METV_MAXSIZE; i++ )
    {    
      cndata->metvals[i] = 0;
    }

  cndata->visits = 0;
  
  
  return cndata;   
}


void cone_cndata_delete(struct cone_cndata* cndata)
{
  free(cndata);
}

void cone_cndata_enter(struct cone_cndata* cndata, struct cone_metv* metv)
{

  timebasestruct_t t;

  cone_metv_read(metv, cndata->enter_vals);

  cndata->enter_time = 0.0;
  read_real_time(&t, TIMEBASE_SZ);
  time_base_to_time(&t, TIMEBASE_SZ);
  cndata->enter_time = t.tb_high + (t.tb_low * 1.0e-9);  
 
  cndata->visits++;
}


void cone_cndata_exit(struct cone_cndata* cndata, struct  cone_metv* metv)
{
  static unsigned long long  exit_vals[CONE_METV_MAXSIZE];
  static double              exit_time;
  static double              exit_u_time;
  static double              exit_s_time; 
  static int                        i;  
  timebasestruct_t           t; 


  cone_metv_read(metv, exit_vals);

  exit_time = 0.0;

  read_real_time(&t, TIMEBASE_SZ);
  time_base_to_time(&t, TIMEBASE_SZ);
  exit_time = t.tb_high + (t.tb_low * 1.0e-9);
 

  

  for ( i = 0; i < cndata->nmets; i++)
    {
      cndata->metvals[i] = cndata->metvals[i] + (exit_vals[i] - cndata->enter_vals[i]);
    }
  

  cndata->time = cndata->time + (exit_time - cndata->enter_time);  
}

int cone_cndata_cnode_id(struct cone_cndata* cndata)
{
  return cndata->cnode_id;
}


unsigned long long cone_cndata_get_metvalue(struct cone_cndata* cndata, int i)
{
  return cndata->metvals[i];
}


void cone_cndata_fprintf(FILE* stream, struct cone_cndata* cndata)
{
  int   i; 
  
  fprintf(stream, 
	  "CNDATA %2d", 
	  cndata->cnode_id); 

  for ( i = 0; i < cndata->nmets; i++)
    fprintf(stream, " %10llu ", cndata->metvals[i]);  

  fprintf(stream, 
	  " %5e %2d",  
	  cndata->time,          
	  cndata->visits ); 

  fprintf(stream, "\n");
}

int cone_cndata_num_stdmets()
{
  return 2; 
}

void cone_cndata_fprintf_stdmets(FILE* stream)
{
  int i ;

  i = cone_metv_size(); 
  fprintf(stream,"METRIC %d TIME sec incl Wall clock time\n",i++);
  fprintf(stream,"METRIC %d VISITS occ excl Number of visits\n",i++);
}
