/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#include "cone_error.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#define CONE_MSG_PFIX  "CONE"
#define CONE_MSG_SIZE  4096

static int cone_is_verbose()
{
  char* tmp = getenv("CONE_VERBOSE");
  if ((tmp != NULL) &&
      strcmp(tmp, "NO") != 0 &&
      strcmp(tmp, "no") != 0 && 
      strcmp(tmp, "No") != 0 && 
      strcmp(tmp, "0") != 0)
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

static void cone_print_msg(const char* fmt, va_list az)
{
  char buffer[CONE_MSG_SIZE];
  
  sprintf(buffer, "%s:-- ", CONE_MSG_PFIX);
  vsprintf(buffer + strlen(buffer), fmt, az);
  fprintf(stderr, "%s\n", buffer);
  fflush(NULL);
  return;
}

void cone_error()
{
  perror(CONE_MSG_PFIX);
  fprintf(stderr,"error\n");
  fflush(stderr);
  exit(EXIT_FAILURE);
}

void cone_error_msg(const char* fmt, ...)
{
  va_list ap;

  va_start(ap, fmt);
  cone_print_msg(fmt, ap);
  exit(EXIT_FAILURE);
}

void cone_warning(const char* fmt, ...)
{
  va_list ap;

  va_start(ap, fmt);
  cone_print_msg(fmt, ap);
  return;
}

void cone_cntl_msg(const char* fmt, ...)
{   
  va_list ap;

  if (cone_is_verbose())
    {
      va_start(ap, fmt);
      cone_print_msg(fmt, ap);
      return;
    }
}






