/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "cone_func.h"
#include "cone_error.h"

#include <string.h>
#include <stdlib.h>

#include <stdio.h>

struct cone_func
{
  long id;
  char* name;
  long begln;
  long endln;
  long mod_id;
  long ncsites;
  long* csite_idv;
  long tag;
};

struct cone_func*   
cone_func_create(char* func_spec)
{
  char* name;
  int i;
  char* tmp;
  
  struct cone_func* func;
 
  
  func =  malloc(sizeof(struct cone_func));
  if ( func == NULL )
    cone_error();

  if ( strcmp(strtok(func_spec, " "), "FUNCTION") != 0 )
    cone_error_msg("invalid function-specification string");
  
  func->id = atol(strtok(NULL, " "));

  name = strtok(NULL, " ");
  func->name = (char*) malloc(strlen(name) + 1);
  if ( func->name == NULL )
    cone_error();
  memcpy(func->name, name, strlen(name) + 1);
  
  func->begln   = atol(strtok(NULL, " "));
  func->endln   = atol(strtok(NULL, " "));
  func->mod_id  = atol(strtok(NULL, " "));
  func->ncsites = atol(strtok(NULL, " "));
  
  if ( func->ncsites > 0 )
    {
      func->csite_idv = (long*) calloc(func->ncsites, sizeof(long));
      if ( func->csite_idv == NULL )
	cone_error();
    }
  else
    {
      func->csite_idv = NULL;
    }
  
  for ( i = 0; i < func->ncsites; i++)
    func->csite_idv[i] = atol(strtok(NULL, " "));

  func->tag = -1;

  return func;
}

void              
cone_func_delete(struct cone_func* func)
{
  free(func->name);
  free(func->csite_idv);
  free(func);
}

long              
cone_func_id(struct cone_func* func)
{
  return func->id;
}

const char*      
cone_func_name(struct cone_func* func)
{
  return func->name;
}

long              
cone_func_begln(struct cone_func* func)
{
  return func->begln;
}

long              
cone_func_endl(struct cone_func* func)
{
  return func->endln;
}

long              
cone_func_mod_id(struct cone_func* func)
{
  return func->mod_id;
}

long              
cone_func_num_funcs(struct cone_func* func)
{
  return func->ncsites;
}

long  
cone_func_csite_id(struct cone_func* func, long i)
{
  return func->csite_idv[i];
}

long              
cone_func_get_tag(struct cone_func* func)
{
  return func->tag;
}

void              
cone_func_set_tag(struct cone_func* func, long tag)
{
  func->tag = tag;
}

void             
cone_func_fprintf(FILE* stream, struct cone_func* func)
{
  int i;
  
  fprintf(stream, 
	  "FUNCTION %d %s %d %d %d %d", 
	  func->id, 
	  func->name, 
	  func->begln, 
	  func->endln, 
	  func->mod_id, 
	  func->ncsites);  
  for ( i = 0; i < func->ncsites; i++ )
    fprintf(stream, " %d", func->csite_idv[i]);
  fprintf(stream,"\n");
 /*  fprintf(stream, " tag = %d\n", func->tag); */
}






