/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "cone_metv.h"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "papivi.h"
#include "cone_error.h"
#include "cone_metric.h"

#define MAX_LINE_LENGTH 8192

extern int rank;

/* CONE -> PAPI event map */
static struct {
  char* event_name;
  int   papi_code;
} cone2papi[] = {

  /** cache level 1 **/

  /* cache level 1 accesses */

  { "L1_ACCESS", PAPI_L1_TCA },
  { "L1_I_ACCESS", PAPI_L1_ICA },
  { "L1_D_ACCESS", PAPI_L1_DCA },
  
  /* cache level 1 reads */
  
  { "L1_READ", PAPI_L1_TCR },
  { "L1_I_READ", PAPI_L1_ICR },
  { "L1_D_READ", PAPI_L1_DCR },
  
  /* cache level 1 writes */
  
  { "L1_WRITE", PAPI_L1_TCW },
  { "L1_I_WRITE", PAPI_L1_ICW },
  { "L1_D_WRITE", PAPI_L1_DCW },
  
  /* cache level 1 hits */
  
  { "L1_HIT", PAPI_L1_TCH },
  { "L1_I_HIT", PAPI_L1_ICH },
  { "L1_D_HIT", PAPI_L1_DCH },
  
  /* cache level 1 misses */
  
  { "L1_MISS", PAPI_L1_TCM },
  { "L1_I_MISS", PAPI_L1_ICM },
  { "L1_D_MISS", PAPI_L1_DCM },
  
  /** cache level 2 **/

  /* cache level 2 accesses */

  { "L2_ACCESS", PAPI_L2_TCA },
  { "L2_I_ACCESS", PAPI_L2_ICA },
  { "L2_D_ACCESS", PAPI_L2_DCA },
  
  /* cache level 2 reads */
  
  { "L2_READ", PAPI_L2_TCR },
  { "L2_I_READ", PAPI_L2_ICR },
  { "L2_D_READ", PAPI_L2_DCR },
  
  /* cache level 2 writes */
  
  { "L2_WRITE", PAPI_L2_TCW },
  { "L2_I_WRITE", PAPI_L2_ICW },
  { "L2_D_WRITE", PAPI_L2_DCW },
  
  /* cache level 2 hits */
  
  { "L2_HIT", PAPI_L2_TCH },
  { "L2_I_HIT", PAPI_L2_ICH },
  { "L2_D_HIT", PAPI_L2_DCH },
  
  /* cache level 2 misses */
  
  { "L2_MISS", PAPI_L2_TCM },
  { "L2_I_MISS", PAPI_L2_ICM },
  { "L2_D_MISS", PAPI_L2_DCM },

  /** cache level 3 **/

  /* cache level 3 accesses */

  { "L3_ACCESS", PAPI_L3_TCA },
  { "L3_I_ACCESS", PAPI_L3_ICA },
  { "L3_D_ACCESS", PAPI_L3_DCA },
  
  /* cache level 3 reads */
  
  { "L3_READ", PAPI_L3_TCR },
  { "L3_I_READ", PAPI_L3_ICR },
  { "L3_D_READ", PAPI_L3_DCR },
  
  /* cache level 3 writes */
  
  { "L3_WRITE", PAPI_L3_TCW },
  { "L3_I_WRITE", PAPI_L3_ICW },
  { "L3_D_WRITE", PAPI_L3_DCW },
  
  /* cache level 3 hits */
  
  { "L3_HIT", PAPI_L3_TCH },
  { "L3_I_HIT", PAPI_L3_ICH },
  { "L3_D_HIT", PAPI_L3_DCH },
  
  /* cache level 3 misses */
  
  { "L3_MISS", PAPI_L3_TCM },
  { "L3_I_MISS", PAPI_L3_ICM },
  { "L3_D_MISS", PAPI_L3_DCM },
  
  /** TLB **/
  
  /* TLB misses */

  { "TLB_MISS", PAPI_TLB_TL },
  { "TLB_I_MISS", PAPI_TLB_IM },
  { "TLB_D_MISS", PAPI_TLB_DM },
  
  /** instructions **/
  
  { "INSTRUCTION", PAPI_TOT_INS },
  { "INTEGER", PAPI_INT_INS },
  { "FP_INS", PAPI_FP_INS },
  { "FP_SQRT_INS", PAPI_FSQ_INS},
  { "FP_DIV_INS", PAPI_FDV_INS},
  { "LOAD_STORE", PAPI_LST_INS },
  { "LOAD", PAPI_LD_INS },
  { "STORE", PAPI_SR_INS },
  { "COND_STORE", PAPI_CSR_TOT },
  { "COND_STORE_SUCCESS", PAPI_CSR_SUC },
  { "COND_STORE_UNSUCCESS", PAPI_CSR_FAL },
  { "COND_BRANCH", PAPI_BR_CN },
  { "COND_BRANCH_TAKEN", PAPI_BR_TKN },
  { "COND_BRANCH_NOTTAKEN", PAPI_BR_NTK },
  { "COND_BRANCH_PRED", PAPI_BR_PRC },
  { "COND_BRANCH_MISPRED", PAPI_BR_MSP },

  /** operations **/
  
  { "FP_OPS", PAPI_FP_OPS },
  
  /** cycles **/
  
  { "CYCLES", PAPI_TOT_CYC },
  
  /** idle units **/
  
  { "INTEGER_UNIT_IDLE", PAPI_FXU_IDL },
  { "FLOAT_UNIT_IDLE", PAPI_FPU_IDL },
  { "BRANCH_UNIT_IDLE", PAPI_BRU_IDL },
  { "LOADSTORE_UNIT_IDLE", PAPI_LSU_IDL },

  /** stalls **/
  
  { "STALL_MEMORY_ACCESS", PAPI_MEM_SCY },
  { "STALL_MEMORY_READ", PAPI_MEM_RCY },
  { "STALL_MEMORY_WRITE", PAPI_MEM_WCY },
  
  /** termination of list **/

  { NULL, 0 }
}; 

struct cone_metv
{

  int EventSet;
};

/* global variables */
static struct cone_metric* metricv[CONE_METV_MAXSIZE];
static char   errstring[PAPI_MAX_STR_LEN];
static int    nmetrics = 0;

/* PAPI-specific error message */

void cone_metv_error(int errcode)
{
  char   errstring[PAPI_MAX_STR_LEN];

  PAPIvi_perror(errcode, errstring, PAPI_MAX_STR_LEN);
  cone_error_msg("%s", errstring);
}

/* test whether requested event combination valid */

void cone_metv_test()
{
  int i;
  int retval;
  int EventSet = PAPI_NULL;
  
  /* create event set */
  retval = PAPIvi_create_eventset(&EventSet);
  if ( retval != PAPI_OK)
    cone_metv_error(retval);
  
  for ( i = 0; i < nmetrics; i++ )
    {
      /* add event to event set */
      retval = PAPIvi_add_event(EventSet, cone_metric_papi_code(metricv[i]));
      if ( retval != PAPI_OK )
	cone_metv_error(retval);
      cone_cntl_msg("Event %s added to event set", cone_metric_name(metricv[i]));
    }

  retval = PAPIvi_cleanup_eventset(EventSet);
  if ( retval != PAPI_OK )
    cone_metv_error(retval);
  
  retval = PAPIvi_destroy_eventset(&EventSet);
  if ( retval != PAPI_OK )
    cone_metv_error(retval);

  cone_cntl_msg("Event set tested OK");
}

int cone_metv_open()
{
  FILE* file;
  char*  env; 
  char*  var; 
  char*  token;
  char  tmp[PAPI_MAX_STR_LEN];
  int   retval;
  int   i, j;
  struct metric* event;
  int   EventSet = PAPI_NULL;
  
  nmetrics = 0; 

  /* read environment variable "CONE_METRICS". Return if unset. */
  env = getenv("CONE_METRICS");
  if ( env == NULL )
    return nmetrics;
  
  var = (char*) calloc(strlen(env) + 1, sizeof(char));
  if ( var == NULL )
    cone_error();      
  memcpy(var, env, strlen(env) + 1);

  /* initialize PAPI */
  retval = PAPIvi_library_init(PAPI_VER_CURRENT);

  if ( retval != PAPI_VER_CURRENT )
    cone_metv_error(retval);

  /* read counter names */
  token = strtok(var, ":");
  
  while ( token )
    {
      int j = 0;
      while ( cone2papi[j].event_name != NULL )
	{
	  if ( strcmp(cone2papi[j].event_name, token) == 0 )
	    {
	      metricv[nmetrics] = cone_metric_create(nmetrics, token, cone2papi[j].papi_code);
	      break;
	    }
	  j++;
	}
      if ( cone2papi[j].event_name == NULL )
	cone_error_msg("Unknown counter %s", token);

      nmetrics++;
      
      if ( nmetrics > CONE_METV_MAXSIZE )
	cone_error_msg("Number of counters exceeds allowed maximum of %d", CONE_METV_MAXSIZE);

      token = strtok(NULL, ":");
   }

  /* free variable var */
  free(var);
  
  /* Check whether event combination is valid. This is done here to
     avoid errors when creating the event set for each thread, which
     would multiply the error messages. */
  if ( nmetrics != 0 )
    cone_metv_test();
    
  return nmetrics;
}

void cone_metv_close()
{
  int i;
  
  for ( i = 0; i < nmetrics; i++ )
    cone_metric_free(metricv[i]);
}

void cone_metv_thread_init(int (*id_fn)(void))
{
  int retval;

  if ( nmetrics == 0 )
    return;

  /* retval = PAPIvi_thread_init((unsigned long (*)(void))(id_fn), 0);  */
  if ( retval != PAPI_OK)
    cone_metv_error(retval);
  cone_cntl_msg("PAPI thread support initialized");
}


int cone_metv_size()
{
  return nmetrics;
}

double cone_metv_clckrt()
{
  const PAPIvi_hw_info_t* hwinfo = NULL;

  hwinfo = PAPIvi_get_hardware_info(); 
  if ( hwinfo == NULL)
    cone_error_msg("Failed to access PAPI harware info");
  cone_cntl_msg("Clock rate: %f MHz", hwinfo->mhz);

  return hwinfo->mhz * 1000000.0;
}

struct cone_metric* cone_metv_metric(int i)
{
  return metricv[i];
}

struct cone_metv* cone_metv_create()
{
  struct cone_metv* metv;
  int retval, i;

  if ( nmetrics == 0 )
    return NULL;

  metv = (struct cone_metv*) malloc(sizeof(struct cone_metv));
  if ( metv == NULL )
    cone_error();
  
  /* create event set */
  retval = PAPIvi_create_eventset(&metv->EventSet);
  if ( retval != PAPI_OK)
    cone_metv_error(retval);
  
  for ( i = 0; i < nmetrics; i++ )
    {
      /* add event to event set */
      retval = PAPIvi_add_event(metv->EventSet, cone_metric_papi_code(metricv[i]));
      if ( retval != PAPI_OK )
	cone_metv_error(retval);
    }

  retval = PAPIvi_start(metv->EventSet);
  if ( retval != PAPI_OK )
    cone_metv_error(retval);
  cone_cntl_msg("Counters started");

  return metv;
}

void cone_metv_free(struct cone_metv* metv)
{
  int retval;
  long_long papi_vals[CONE_METV_MAXSIZE];

  if ( metv == NULL )
    return;

  retval = PAPIvi_stop(metv->EventSet, papi_vals);
  if ( retval != PAPI_OK )
    cone_metv_error(retval);

  retval = PAPIvi_cleanup_eventset(metv->EventSet);
  if ( retval != PAPI_OK )
    cone_metv_error(retval);

  retval = PAPIvi_destroy_eventset(&metv->EventSet);
  if ( retval != PAPI_OK )
    cone_metv_error(retval);
    
  free(metv);

  cone_cntl_msg("Counters stopped");
}

void cone_metv_read(struct cone_metv* metv, unsigned long long values[])
{
  int retval, i;

  if ( metv == NULL )
    return;

  if (sizeof(long_long) == sizeof(unsigned long long))
    {
      retval = PAPIvi_read(metv->EventSet, (long_long*) values);
    }
  else
    {
      int i;
      long_long papi_vals[CONE_METV_MAXSIZE];
      retval = PAPIvi_read(metv->EventSet, papi_vals);
      for (i = 0; i < nmetrics; i++)
	values[i] += (unsigned long long) papi_vals[i];
    }
     
  if ( retval != PAPI_OK )
    cone_metv_error(retval);
}




