/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "cone_mod.h"
#include "cone_error.h"

#include <string.h>
#include <stdlib.h>

struct cone_mod
{
  long id;
  char* name;
  long nfuncs;
  long* func_idv;
  long tag;
};

struct cone_mod*   
cone_mod_create(char* mod_spec)
{
  char* name;
  int i;
  char* tmp;
  struct cone_mod* mod;
  mod =  malloc(sizeof(struct cone_mod)); 
  if ( mod == NULL )
    cone_error();

  if ( strcmp(strtok(mod_spec, " "), "MODULE") != 0 )
    cone_error_msg("invalid module-specification string");
  
  mod->id = atol(strtok(NULL, " "));

  name = strtok(NULL, " ");
  mod->name = (char*) malloc(strlen(name) + 1);
  if ( mod->name == NULL )
    cone_error();
  memcpy(mod->name, name, strlen(name) + 1);
  
  mod->nfuncs = atol(strtok(NULL, " "));
  
  if ( mod->nfuncs > 0 )
    {
      mod->func_idv = (long*) calloc(mod->nfuncs, sizeof(long));
      if ( mod->func_idv == NULL )
	cone_error();
    }
  else
    {
      mod->func_idv == NULL;
    }
  
  for ( i = 0; i < mod->nfuncs; i++)
    mod->func_idv[i] = atol(strtok(NULL, " "));

  mod->tag = -1;

  return mod;
}

void              
cone_mod_delete(struct cone_mod* mod)
{
  free(mod->name);
  free(mod->func_idv);
  free(mod);
}

long              
cone_mod_id(struct cone_mod* mod)
{
  return mod->id;
}

const char*      
cone_mod_name(struct cone_mod* mod)
{
  return mod->name;
}

long              
cone_mod_num_funcs(struct cone_mod* mod)
{
  return mod->nfuncs;
}

long  
cone_mod_func_id(struct cone_mod* mod, long i)
{
  return mod->func_idv[i];
}

long              
cone_mod_get_tag(struct cone_mod* mod)
{
  return mod->tag;
}

void              
cone_mod_set_tag(struct cone_mod* mod, long tag)
{
  mod->tag = tag;
}

void             
cone_mod_fprintf(FILE* stream, struct cone_mod* mod)
{
  int i;
  
  fprintf(stream, "MODULE %d %s %d", mod->id, mod->name, mod->nfuncs);  
  for ( i = 0; i < mod->nfuncs; i++ )
    fprintf(stream, " %d", mod->func_idv[i]);

  fprintf(stream,"\n");
 /*  fprintf(stream, " tag = %d\n", mod->tag); */
}
