/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "cone_probes.h"

#include <stdio.h> 

#ifdef CONE_MPI 
#include <mpi.h>
#endif

#include "cone_prog.h"
#include "cone_cnode.h" 
#include "cone_metv.h"
#include "cone_cg.h"
#include "cone_error.h"


static struct cone_prog*  prog      = NULL;
static struct cone_cnode* curr_node = NULL;
static struct cone_cnode* root_node = NULL;
static struct cone_metv*  metv      = NULL; 

static int my_rank;
static int num_ranks;

static int main_id;

static int main_left = 0;

void cone_init()
{
  cone_metv_open() ;
  prog = cone_prog_create(CONE_APPL_DESCR_FILE) ;
  root_node = cone_prog_cnode(prog,0);
  atexit(cone_term);
}


void cone_start(int func_id)
{ 
  main_id = func_id;
  cone_cg_init(prog);
  cone_enter(func_id);
}

void cone_term()
{
  if ( !main_left ) 
    cone_exit(main_id);
  cone_cg_term();
  cone_prog_delete(prog);
}

void cone_enter(int func_id)
{
  if ( func_id == main_id )
    {
      cone_before(cone_func_csite_id(cone_prog_func(prog,func_id),0));
      curr_node = root_node;
      cone_cg_enter(cone_cnode_id(curr_node));
    }
}

void cone_exit(int func_id)
{
  cone_cg_exit(cone_cnode_id(curr_node));
  curr_node = cone_prog_cnode(prog,cone_cnode_parent_id(curr_node));
  if ( func_id == main_id )
    main_left = 1;
}

void cone_before(int csite_id)
{
  if ( cone_func_id(cone_prog_func(prog, cone_csite_callee_id(cone_prog_csite(prog, csite_id)))) != main_id )
    {
      curr_node = cone_prog_cnode(prog, cone_cnode_child_id(curr_node, cone_csite_index(cone_prog_csite(prog, csite_id))));
      cone_cg_enter(cone_cnode_id(curr_node));
    }
}

void cone_after(int csite_id)
{
  cone_cg_exit(cone_cnode_id(curr_node));
  curr_node = cone_prog_cnode(prog, cone_cnode_parent_id(curr_node));
}

void cone_after_mpi_init()
{
#ifdef CONE_MPI 
  MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);
  MPI_Comm_size(MPI_COMM_WORLD, &num_ranks);

  cone_cg_after_mpi_init(my_rank,num_ranks);
#endif
}

void cone_before_mpi_finalize()
{
#ifdef CONE_MPI 
#endif
}




